<?php
	
	require 'bugzilla_aggregator/config.php';
	require 'bugzilla_aggregator/class.Curler.php';
	require 'bugzilla_aggregator/class.DB.php';
	
	class BugzillaToDB {
		/********************** PRIVATE ***********************/
		private static function _isValidResponse($aResponse) {
			return $aResponse && isset($aResponse['bugs']) && $aResponse['bugs'];
		}
		private static function _runQuery($aQuery) {
			if (!defined('BUGZILLA_URL'))  { 
				die('BUGZILLA_URL is undefined in wp-config.php');  
			}
			$oRequest = xmlrpc_encode_request(
	            'Bug.search',
	            $aQuery,
	            array(
	                'escaping' 	=> array('markup'),
	                'encoding' 	=> 'utf-8'
	            )
	        );
			$oCurler = new Curler;
	        $oCurler->post(
				BUGZILLA_URL.'/xmlrpc.cgi', 
				$oRequest,
	            array(
	                'return_headers'	=> true,
	                'headers'           => array('Content-type: text/xml;charset=UTF-8'),
	                'authenticate'      => null,
	                'ssl_verify_peer' 	=> BUGZILLA_IN_DEV_MODE
	        	)
	        );
	        $aResponse = $oCurler->response_content();
	        $aResponse = xmlrpc_decode($aResponse);
			return $aResponse;
		}
		/********************** PUBLIC ************************/
		public static function getUsers() {
			$aUsers = DB::selectColumn("SELECT user_email FROM `".WP_USERS_TABLE."` WHERE 1");
			return $aUsers;
		}
		public static function getAggregatedResultsForUser($sUserEmail) {
			$n30DaysInSeconds = 30 * 24 * 60 * 60;
			$aAggregatedResults = array(
				'user'						=> $sUserEmail,
				'bugs_opened'				=> 0,
				'bugs_opened_in_30_days'	=> 0,
				'bugs_confirmed'			=> 0,
				'bugs_verified'				=> 0
			);
			
			/////// QUERY 1 ///////
			$aQuery = array(
				'creator' => $sUserEmail
			);
			$aResult = self::_runQuery($aQuery);
			if (self::_isValidResponse($aResult)) {
				foreach ($aResult['bugs'] as $aBug) {
					$aAggregatedResults['bugs_opened'] ++;
					if ($aBug['creation_time']->timestamp + $n30DaysInSeconds >= time()) {
						$aAggregatedResults['bugs_opened_in_30_days'] ++;
					}
				}
			}
			
			//////// QUERY 2 ///////
			$aQuery = array(
				'assigned_to' 	=> $sUserEmail,
				'is_confirmed'	=> 1
			);
			$aResult = self::_runQuery($aQuery);
			if (self::_isValidResponse($aResult)) {
				foreach ($aResult['bugs'] as $aBug) {
					$aAggregatedResults['bugs_confirmed'] ++;
				}
			}
			
			/////// QUERY 3 ///////
			$aQuery = array(
				'qa_contact' 	=> $sUserEmail,
				'status'		=> 'VERIFIED'
			);
			$aResult = self::_runQuery($aQuery);
			if (self::_isValidResponse($aResult)) {
				foreach ($aResult['bugs'] as $aBug) {
					$aAggregatedResults['bugs_verified'] ++;
				}
			}
			return $aAggregatedResults;
		}
		
		public static function updateDatabase() {
			$aUsers = self::getUsers();
			foreach ($aUsers as $sUser) {
				$aRecord = self::getAggregatedResultsForUser($sUser);
				DB::insertArray(BUGZILLA_TABLE, $aRecord, true);
			}
		}
	}
	
	function debug_show($s) {
		echo '<pre  style="border: 1px solid black; background-color: #FF9999;">'.print_r($s, 1).'</pre>';
	}
	
	
	BugzillaToDB::updateDatabase();

?>