<?php

/**
 * Bugzilla library.
 *
 * @see http://www.bugzilla.org/docs/tip/en/html/api/Bugzilla/WebService/Bug.html
 *
 *
 * @package    Bugzilla
 * @author     skeen@mozilla.org
 */

require_once('class.Curler.php');

class BugzillaClient {
    private $error_message;
    private $curler;

    private $bz_id		= null;
    private $bz_token	= null;
	private $bz_url		= null;

    private $httpauth_credentials;

    public function  __construct($bz_url, $httpauth_credentials = null) {
		$this->bz_url = $bz_url;
        $this->httpauth_credentials = $httpauth_credentials;
        $this->bz_id 				= $_SESSION['bz_id'];
        $this->bz_token 			= $_SESSION['bz_token'];
        $this->curler 				= new Curler();
    }
    /**
     * After a failed interaction w/ Bugzilla, this will contain the error message
     * that was returned.
     * @return string
     */
    public function error($sError) {
        die($sError);
    }

    /**
     * Atempt to authenticate to the Bugzilla system
     * If an error is returned it is set to $this->error_message
     *
     * @param string $username
     * @param string $password
     * @return boolean
     */
    public function login($bugzilla_url, $username, $password) {
		$this->bz_url = $bugzilla_url;
        $login_success = false;
        $request = xmlrpc_encode_request(
                "User.login",
                array(
                'login'     => $username,
                'password'  => $password
                )
        );
        $response = xmlrpc_decode_request($this->call($request), $request);
        if(empty($response)) {
            $this->error("There was an unexpected error while logging into Bugzilla");
        } else if (isset($response['faultString'])) {
            $this->error($response['faultString']);
        } else {
            $_SESSION['bz_id'] 	  = $this->bz_id;
            $_SESSION['bz_token'] = $this->bz_token;
            $login_success 		  = true;
        }

        return $login_success;
    }
    public function logout() {
        unset($_SESSION['bz_id']);
		unset($_SESSION['bz_token']);
    }
    public function authenticated() {
        return $this->bz_id && $this->bz_token;
    }
    /**
     *
     * @param sting $xml
     * @return
     */
    public function call($xml) {
        $bugzilla_server = $this->bz_url;
        $bugzilla_xmlrpc = "/xmlrpc.cgi";
        $additional_headers = array('Content-type: text/xml;charset=UTF-8');
        if ( $this->bz_id && $this->bz_token ) {
            $cookie = "Cookie: Bugzilla_login="
                    ."{$this->bz_id}; Bugzilla_logincookie={$this->bz_token}";
            ("Setting Cookie Header: [$cookie]");
            $additional_headers[] = $cookie;
        }
        $this->curler->post($bugzilla_server.$bugzilla_xmlrpc, $xml,
                array(
                'return_headers'=>true,
                'headers' => $additional_headers,
                'authenticate' => $this->httpauth_credentials,
                // when not in production have curl ignore ssl warings which are
                // most ofter due to self signed certs
                'ssl_verify_peer' => BUGZILLA_IN_DEV_MODE
                )
        );
        $set_cookies = $this->curler->response_headers('Set-Cookie');
        $response = $this->curler->response_content();

        if($set_cookies) {
            $bz_creds = array();
            foreach ($set_cookies as $set_cookie) {
                $match = null;
                preg_match('/(Bugzilla_login|Bugzilla_logincookie)?=(.*?);/i', $set_cookie, $match);
                if(isset($match[2])) {
                    if(strtolower($match[1])=='bugzilla_login') {
                        $this->bz_id = $match[2];
                    } else if(strtolower($match[1])=='bugzilla_logincookie') {
                        $this->bz_token = $match[2];
                    }
                }
            }
        }
        return $response;
    }
	/**
     *
     * @param sting $xml
     * @return
     */
    public function call_anonymous($xml) {
        $bugzilla_server = $this->bz_url;
        $bugzilla_xmlrpc = "/xmlrpc.cgi";
        $additional_headers = array('Content-type: text/xml;charset=UTF-8');
       /* if ( $this->bz_id && $this->bz_token ) {
            $cookie = "Cookie: Bugzilla_login="
                    ."{$this->bz_id}; Bugzilla_logincookie={$this->bz_token}";
            ("Setting Cookie Header: [$cookie]");
            $additional_headers[] = $cookie;
        }*/
        $this->curler->post($bugzilla_server.$bugzilla_xmlrpc, $xml,
                array(
                'return_headers'=>true,
                'headers' => $additional_headers,
                'authenticate' => $this->httpauth_credentials,
                // when not in production have curl ignore ssl warings which are
                // most ofter due to self signed certs
                'ssl_verify_peer' => BUGZILLA_IN_DEV_MODE
                )
        );
        //$set_cookies = $this->curler->response_headers('Set-Cookie');
        $response = $this->curler->response_content();

      /*  if($set_cookies) {
            $bz_creds = array();
            foreach ($set_cookies as $set_cookie) {
                $match = null;
                preg_match('/(Bugzilla_login|Bugzilla_logincookie)?=(.*?);/i', $set_cookie, $match);
                if(isset($match[2])) {
                    if(strtolower($match[1])=='bugzilla_login') {
                        $this->bz_id = $match[2];
                    } else if(strtolower($match[1])=='bugzilla_logincookie') {
                        $this->bz_token = $match[2];
                    }
                }
            }
        }*/
        return $response;
    }

}